/*

% Description of my shader.
% Second line of description for my shader.

keywords: material classic

date: YYMMDD

*/

/////////////////////////////////////////////
//
//		GLOBAL VALUE-COMMON
//
/////////////////////////////////////////////
float4x4 WorldViewProj	: WorldViewProjection;
float4x4 matWorldInv	: WorldInverse;
float4x4 matProj		: Projection;
float4x4 matView		: View;
float4x4 matWorld		: World;
float4	g_color0		: COLOR;
float4	g_color1		: COLOR;
float4	g_lightDir		: POSITION;
float4	g_offset;
float4x4 g_vlp;

texture	g_texture;
texture	g_shadowBuf;

/////////////////////////////////////////////
//
//		GLOBAL VALUE-TOON
//
/////////////////////////////////////////////
texture ShadeTexture;
sampler ShadeSampler = sampler_state{
	Texture = (ShadeTexture);
	MinFilter = NONE;
	MagFilter = NONE;
	MipFilter = NONE;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

/////////////////////////////////////////////
//
//		TEXTURE SAMPLER
//
/////////////////////////////////////////////
sampler TextureSampler = 
sampler_state{
	Texture = <g_texture>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	
	AddressU = Clamp;
	AddressV = Clamp;
};

sampler ShadowBufSampler = 
sampler_state{
	Texture = <g_shadowBuf>;
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	
	AddressU = Clamp;
	AddressV = Clamp;
};

///////////////////////////////////////////
//	VS_INPUT/OUTPUT
///////////////////////////////////////////
struct VS_INPUT{
	float3 Pos	: POSITION;
	float3 Norm	: NORMAL;
	float2 Tex	: TEXCOORD0;
	float4 Col	: COLOR0;
};

struct VS_OUTPUT{
	//COMMON-SHADOW
	float4 Pos	: POSITION;
	float4 Col	: COLOR0;
	float2 Tex	: TEXCOORD0;
	float4 Light: TEXCOORD1;
	//TOON
	float2 Shade: TEXCOORD2;
	//FOG
	float3 PosW	: TEXCOORD3;
	float3 PosV	: TEXCOORD4;
};

///////////////////////////////////////////
//	PS_INPUT/OUTPUT
///////////////////////////////////////////
struct PS_INPUT{
	float4 Col	: COLOR0;
	float2 Tex	: TEXCOORD0;
	float4 Light: TEXCOORD1;
	//TOON
	float2 Shade: TEXCOORD2;
	//FOG
	float3 PosW	: TEXCOORD3;
	float3 PosV	: TEXCOORD4;
};


VS_OUTPUT mainVS(VS_INPUT In) {
	VS_OUTPUT Out = (VS_OUTPUT)0;
	
//>>>>>>>>>>>		SHADOW			>>>>>>>>>>>>>>>>>>>>>>
	float4 world_pos;
	world_pos = mul( float4(In.Pos,1.0f),matWorld );
	
	//vertex position transform
	Out.Pos = mul( float4(In.Pos,1.0f),WorldViewProj );
	Out.Tex = In.Tex;
	
	//Color-tmp code
	Out.Col = g_color0;
	
	//calc vertex position from light
	Out.Light = mul(float4(In.Pos,1.0f),g_vlp);
//<<<<<<<<<<<		SHADOW			<<<<<<<<<<<

//>>>>>>>>>>>>>>	TOON	>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	float3 lightVec 	= g_lightDir.xyz;
	float3 tmpNormal	= mul( In.Norm,(float3x3)matWorld );
	Out.Shade			= max( dot(tmpNormal,lightVec), 0 );
	
//<<<<<<<<<<<<<		TOON	<<<<<<<<<<<<<<<<<<<<<<<<<<<<

//>>>>>>>>>>>>>>	FOG	>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	Out.PosW	= world_pos;
	Out.PosV	= mul(world_pos,matView);

//<<<<<<<<<<<<<		FOG	<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	return Out;
}

float calcFogW(float in_pos_y){
	return 	max(0.0, (500.0 - (in_pos_y+100.0))/500.0)*0.7f;
}

//shadeless Pixel shader
float4 shadelessPS( PS_INPUT In,uniform bool b_tex) :COLOR{
	float4 oColor;
	if(b_tex){
		oColor	= In.Col * tex2D( TextureSampler,In.Tex );
	}else{
		oColor	= In.Col;
	}
	
	//>>>>> FOG >>>>>
	float fog_oku;
	fog_oku = max(0.0, (In.PosV.z - 500.0)/250000.0);

	float fog_tate;
//	fog_tate = calcFogW(In.PosW.y);

	oColor.rgb = oColor.rgb + pow(fog_oku, 2.0); 
	
	//<<<<< FOG <<<<<
	
	return oColor;
}

float4 mainPS(PS_INPUT In,uniform bool b_tex) : COLOR {
	float4 oColor;
	//get value from shadow buffer
	float2 shadowBuffUV = 	(0.5 * ( In.Light.xy / In.Light.w )) + float2(0.5,0.5);
	shadowBuffUV.y	= 1.0f - shadowBuffUV.y;
	shadowBuffUV.x	= shadowBuffUV.x + g_offset.x;
	shadowBuffUV.y	= shadowBuffUV.y + g_offset.y;
	float shadow_val = tex2D( ShadowBufSampler,shadowBuffUV ).r;
	
	float2 ofs0,ofs1,ofs2,ofs3;
	float shadow_val0,shadow_val1,shadow_val2,shadow_val3;
	ofs0.x = shadowBuffUV.x + g_offset.x;
	ofs0.y = shadowBuffUV.y + g_offset.y;
	
	ofs1.x = shadowBuffUV.x - g_offset.x;
	ofs1.y = shadowBuffUV.y + g_offset.y;
	
	ofs2.x = shadowBuffUV.x - g_offset.x;
	ofs2.y = shadowBuffUV.y - g_offset.y;
	
	ofs3.x = shadowBuffUV.x + g_offset.x;
	ofs3.y = shadowBuffUV.y - g_offset.y;
	
	shadow_val0 = tex2D( ShadowBufSampler,ofs0 ).r;
	shadow_val1 = tex2D( ShadowBufSampler,ofs1 ).r;
	shadow_val2 = tex2D( ShadowBufSampler,ofs2 ).r;
	shadow_val3 = tex2D( ShadowBufSampler,ofs3 ).r;
	
	shadow_val = (shadow_val+shadow_val0+shadow_val1+shadow_val2+shadow_val3)/5.0f;
	
	if(b_tex){
		oColor	= In.Col * tex2D( TextureSampler,In.Tex );
		oColor = oColor * tex1D( ShadeSampler,In.Shade );
	}else{
		oColor	= In.Col;
	}
	
	// length from light to pixel
	float depth = In.Light.z / In.Light.w;
	
	float maxDepth = 2000.0f;
	float baseDepth = 16000.0f - maxDepth;
	depth = In.Light.z;
	if( depth < baseDepth ){
		depth = 1.0f;
	}else{
		depth = depth - baseDepth;
	}
	depth = depth/maxDepth;

	if( depth - shadow_val > 0.001f ){
		//mix around pixel color
		oColor = float4( oColor.rgb * (0.8f),g_color0.a);
	}
	
	//>>>>> FOG >>>>>
	float fog_oku;
	fog_oku = max(0.0, (In.PosV.z - 500.0)/170000.0);

	float fog_tate;
//	fog_tate = calcFogW(In.PosW.y);

//	oColor.rgb = oColor.rgb + min(0.5f,pow(fog_tate, 2.0)) + pow(fog_oku, 2.0); 
	oColor.rgb = oColor.rgb + pow(fog_oku, 2.0); 
	
	//<<<<< FOG <<<<<
	return oColor;
}

technique technique0 {
	pass p0 {
		//tex enable
		Sampler[0] = ShadeSampler;
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 mainPS(true);
	}
	
	pass p1 {
		//tex disable
		Sampler[0] = ShadeSampler;
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 mainPS(false);
	}
	
	pass p2{
		Sampler[0] = ShadeSampler;
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 shadelessPS(true);
	}
	
	pass p3{
		Sampler[0] = ShadeSampler;
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 shadelessPS(false);
	}
}
